@@ -1,12 +1,10 @@
 import math
 
 from svg_to_gcode import formulas
-from svg_to_gcode.geometry import Vector
+from svg_to_gcode.geometry import Vector, RotationMatrix
 from svg_to_gcode.geometry import Curve
 
 
-# Todo investigate odd behaviour when transform_origin is false. Parsing it without transforming and then tracing to svg
-#  without transforming translates the drawing up a little.
 class EllipticalArc(Curve):
     """The EllipticalArc class inherits from the abstract Curve class and describes an elliptical arc."""
 
@@ -18,13 +16,11 @@ class EllipticalArc(Curve):
         self.center = center
         self.radii = radii
         self.rotation = rotation
-        # Constrain angle within +-360 degrees
-        max_angle = 2*math.pi
-        self.start_angle = formulas.mod_constrain(start_angle, -max_angle, max_angle)
-        self.sweep_angle = formulas.mod_constrain(sweep_angle, -max_angle, max_angle)
+        self.start_angle = start_angle
+        self.sweep_angle = sweep_angle
 
         # Calculate missing data
-        self.end_angle = formulas.mod_constrain(start_angle + sweep_angle, -max_angle, max_angle)
+        self.end_angle = start_angle + sweep_angle
         self.start = self.angle_to_point(self.start_angle)
         self.end = self.angle_to_point(self.end_angle)
 
@@ -38,10 +34,10 @@ class EllipticalArc(Curve):
         angle = formulas.linear_map(self.start_angle, self.end_angle, t)
         return self.angle_to_point(angle)
 
-    def angle_to_point(self, rad):
-        at_origin = Vector(self.radii.x * math.cos(rad), self.radii.y * math.sin(rad))
-        translated = self.center + formulas.rotate(at_origin, self.rotation, False)
-        return translated
+    def angle_to_point(self, angle):
+        transformed_radii = Vector(self.radii.x * math.cos(angle), self.radii.y * math.sin(angle))
+        point = RotationMatrix(self.rotation) * transformed_radii + self.center
+        return point
 
     def derivative(self, t):
         angle = formulas.linear_map(self.start_angle, self.end_angle, t)
