@@ -5,8 +5,8 @@ from typing import List
 
 from svg_to_gcode.geometry import Vector
 from svg_to_gcode.geometry import Line, EllipticalArc, CubicBazier, QuadraticBezier
+from svg_to_gcode.svg_parser import Transformation
 from svg_to_gcode import formulas
-from svg_to_gcode import TOLERANCES
 
 verbose = False
 
@@ -17,25 +17,32 @@ class Path:
     command_lengths = {'M': 2, 'm': 2, 'L': 2, 'l': 2, 'H': 1, 'h': 1, 'V': 1, 'v': 1, 'Z': 0, 'z': 0, 'C': 6, 'c': 6,
                        'Q': 4, 'q': 4, 'S': 4, 's': 4, 'T': 2, 't': 2, 'A': 7, 'a': 7}
 
-    __slots__ = "curves", "initial_point", "current_point", "last_control", "canvas_height", "do_vertical_mirror", \
-                "do_vertical_translate", "draw_move", "transformation"
+    __slots__ = "curves", "initial_point", "current_point", "last_control", "canvas_height", "draw_move", \
+                "transform_origin", "point_transformation"
 
-    def __init__(self, d: str, canvas_height: float, do_vertical_mirror=True, do_vertical_translate=True,
-                 transformation=None):
+    def __init__(self, d: str, canvas_height: float, transform_origin=True, transformation=None):
         self.canvas_height = canvas_height
-        self.do_vertical_mirror = do_vertical_mirror
-        self.do_vertical_translate = do_vertical_translate
-        self.transformation = transformation
+        self.transform_origin = transform_origin
 
         self.curves = []
         self.initial_point = Vector(0, 0)  # type: Vector
         self.current_point = Vector(0, 0)
         self.last_control = None  # type: Vector
 
+        point_transformation = Transformation()
+
+        if self.transform_origin:
+            point_transformation.add_translation(0, canvas_height)
+            point_transformation.add_scale(1, -1)
+
+        if transformation is not None:
+            point_transformation.extend(transformation)
+
+        self.point_transformation = point_transformation
+
         try:
             self._parse_commands(d)
         except Exception as generic_exception:
-            raise generic_exception
             warnings.warn(f"Terminating path. The following unforeseen exception occurred: {generic_exception}")
 
     def __repr__(self):
@@ -82,10 +89,23 @@ class Path:
             if is_numeric:
                 number_str += character
 
+                # if a negative number follows another number, no delimiter is required.
+                # implicitly stated decimals like .6 don't require a delimiter. In either case we add a delimiter.
+                negatives = not is_final and character != 'e' and d[i + 1] == '-'
+                implicit_decimals = not is_final and d[i + 1] == '.' and '.' in number_str
+                if negatives or implicit_decimals:
+                    d = d[:i+1] + ',' + d[i+1:]
+
             # If the character is a delimiter or a command key or the last character, complete the number and save it
             # as an argument
             if is_delimiter or is_command_key or is_final:
                 if number_str:
+                    # In svg form '-.5' can be written as '-.5'. Python doesn't like that notation.
+                    if number_str[0] == '.':
+                        number_str = '0' + number_str
+                    if number_str[0] == '-' and number_str[1] == '.':
+                        number_str = '-0' + number_str[1:]
+
                     command_arguments.append(float(number_str))
                     number_str = ''
 
@@ -104,22 +124,8 @@ class Path:
 
             i += 1
 
-    def _transform_coordinate_system(self, point: Vector):
-        """
-        If both do_vertical_mirror and do_vertical_translate are true, it will transform_origin a point form a coordinate
-        system with the origin at the top-left, to one with origin at the bottom-right.
-        """
-
-        if self.transformation:
-            point = self.transformation.apply_transformation(point)
-
-        if self.do_vertical_mirror:
-            point = Vector(point.x, -point.y)
-
-        if self.do_vertical_translate:
-            point += Vector(0, self.canvas_height)
-
-        return point
+    def _apply_transformations(self, point: Vector):
+        return self.point_transformation.apply_transformation(point)
 
     def _add_svg_curve(self, command_key: str, command_arguments: List[float]):
         """
@@ -153,7 +159,7 @@ class Path:
             start = self.current_point
             end = Vector(x, y)
 
-            line = Line(self._transform_coordinate_system(start), self._transform_coordinate_system(end))
+            line = Line(self._apply_transformations(start), self._apply_transformations(end))
 
             self.current_point = end
 
@@ -180,10 +186,10 @@ class Path:
         # Draw curvy curves
         def absolute_cubic_bazier(control1_x, control1_y, control2_x, control2_y, x, y):
 
-            trans_start = self._transform_coordinate_system(self.current_point)
-            trans_end = self._transform_coordinate_system(Vector(x, y))
-            trans_control1 = self._transform_coordinate_system(Vector(control1_x, control1_y))
-            trans_control2 = self._transform_coordinate_system(Vector(control2_x, control2_y))
+            trans_start = self._apply_transformations(self.current_point)
+            trans_end = self._apply_transformations(Vector(x, y))
+            trans_control1 = self._apply_transformations(Vector(control1_x, control1_y))
+            trans_control2 = self._apply_transformations(Vector(control2_x, control2_y))
 
             cubic_bezier = CubicBazier(trans_start, trans_end, trans_control1, trans_control2)
 
@@ -218,9 +224,9 @@ class Path:
 
         def absolute_quadratic_bazier(control1_x, control1_y, x, y):
 
-            trans_end = self._transform_coordinate_system(self.current_point)
-            trans_new_end = self._transform_coordinate_system(Vector(x, y))
-            trans_control1 = self._transform_coordinate_system(Vector(control1_x, control1_y))
+            trans_end = self._apply_transformations(self.current_point)
+            trans_new_end = self._apply_transformations(Vector(x, y))
+            trans_control1 = self._apply_transformations(Vector(control1_x, control1_y))
 
             quadratic_bezier = QuadraticBezier(trans_end, trans_new_end, trans_control1)
 
@@ -251,71 +257,32 @@ class Path:
 
         # Generate EllipticalArc with center notation from svg endpoint notation.
         # Based on w3.org implementation notes. https://www.w3.org/TR/SVG2/implnote.html
+        # Todo transformations aren't applied correctly to elliptical arcs
         def absolute_arc(rx, ry, deg_from_horizontal, large_arc_flag, sweep_flag, x, y):
-            start = self.current_point
             end = Vector(x, y)
+            start = self.current_point
+
+            radii = Vector(rx, ry)
 
             rotation_rad = math.radians(deg_from_horizontal)
-            max_angle = 2 * math.pi
-            rotation_rad = formulas.mod_constrain(rotation_rad, -max_angle, max_angle)
 
-            # Find and select one of the two possible eclipse centers by undoing the rotation (to simplify the math) and
-            # then re-applying it.
-            rotated_primed_values = (start - end) / 2  # Find the primed_values of the start and the end points.
-            primed_values = formulas.rotate(rotated_primed_values, -rotation_rad, True)  # Undo the ellipse's rotation.
-            px, py = primed_values.x, primed_values.y
+            if abs(start-end) == 0:
+                raise ValueError("start and end points can't be equal")
 
-            # Correct out-of-range radii
-            # ToDo investigate buggy behaviour when sweep angle > 180 deg
-            rx = abs(rx)
-            ry = abs(ry)
-            if rx <= TOLERANCES['operation'] or ry <= TOLERANCES['operation']:
-                return absolute_line(x, y)
+            radii, center, start_angle, sweep_angle = formulas.endpoint_to_center_parameterization(
+                start, end, radii, rotation_rad, large_arc_flag, sweep_flag)
 
-            delta = px**2/rx**2 + py**2/ry**2
+            center = self._apply_transformations(center)
+            radii = Vector(radii.x, -radii.y)
+            rotation_rad *= -1
 
-            if delta > 1:
-                rx *= math.sqrt(delta)
-                ry *= math.sqrt(delta)
-
-            if math.sqrt(delta) > 1:
-                center = Vector(0, 0)
-            else:
-                radicant = ((rx * ry) ** 2 - (rx * py) ** 2 - (ry * px) ** 2) / ((rx * py) ** 2 + (ry * px) ** 2)
-
-                # Find center using w3.org's formula
-                center = math.sqrt(radicant) * Vector((rx * py) / ry, - (ry * px) / rx)
-
-                center *= -1 if large_arc_flag == sweep_flag else 1  # Select one of the two solutions based on flags
-
-            rotated_center = formulas.rotate(center, rotation_rad, False) + (start + end) / 2  # re-apply the rotation
-
-            cx, cy = center.x, center.y
-            u = Vector((px - cx) / rx, (py - cy) / ry)
-            v = Vector((-px - cx) / rx, (-py - cy) / ry)
-
-            start_angle = formulas.angle_between_vectors(Vector(1, 0), u)
-            sweep_angle_unbounded = formulas.angle_between_vectors(u, v)
-            sweep_angle = sweep_angle_unbounded % max_angle
-
-            if not sweep_flag and sweep_angle_unbounded > 0:
-                sweep_angle -= max_angle
-
-            if sweep_flag and sweep_angle_unbounded < 0:
-                sweep_angle += max_angle
-
-            transformed_center = self._transform_coordinate_system(rotated_center)
-            sweep_angle *= -1 if self.do_vertical_mirror else 1
-            start_angle *= 1 if self.do_vertical_mirror else 1
-
-            arc = EllipticalArc(transformed_center, Vector(rx, ry), rotation_rad, start_angle, sweep_angle)
-
-            self.current_point = Vector(x, y)
+            arc = EllipticalArc(center, radii, rotation_rad, start_angle, sweep_angle)
 
+            self.current_point = end
             return arc
 
         def relative_arc(rx, ry, deg_from_horizontal, large_arc_flag, sweep_flag, dx, dy):
-            return absolute_arc(rx, ry, deg_from_horizontal, large_arc_flag, sweep_flag, self.current_point.x + dx, self.current_point.x + dy)
+            return absolute_arc(rx, ry, deg_from_horizontal, large_arc_flag, sweep_flag, self.current_point.x + dx, self.current_point.y + dy)
 
         command_methods = {
             # Only move end point
@@ -350,7 +317,6 @@ class Path:
         try:
             curve = command_methods[command_key](*command_arguments)
         except TypeError as type_error:
-            raise type_error
             warnings.warn(f"Mis-formed input. Skipping command {command_key, command_arguments} because it caused the "
                           f"following error: \n{type_error}")
         except ValueError as value_error:
